package com.ibm.oti.shared;

/*
 * Licensed Materials - Property of IBM,
 *     Copyright IBM Corp. 1998, 2010  All Rights Reserved
 */

/**
 * Shared Class Helper API that stores and finds classes using String tokens.<p>
 * <p>
 * - Description<p>
 * A SharedClassTokenHelper is obtained by calling getSharedClassTokenHelper(ClassLoader) on a SharedClassHelperFactory.<br>
 * The SharedClassTokenHelper is the most simple type of helper, which uses generated String tokens to find and store classes.
 * <p>
 * - Usage<p>
 * The ClassLoader should call findSharedClass after looking in its local cache and asking its parent (if one exists).
 * If findSharedClass does not return null, the ClassLoader should call defineClass on the byte[] returned.<br>
 * The ClassLoader should call storeSharedClass immediately after a class has been defined, but NOT if the class being defined was loaded from the shared cache.<br>
 * The ClassLoader is reponsible for coordinating the creation of token Strings.
 * <p>
 * - Dynamic Cache Updates<p>
 * Since the shared cache is persistent beyond the lifetime of a JVM, classes in the shared cache may become out-of-date ("stale").
 * Using this helper, it is entirely the responsibility of the ClassLoader to ensure that cache entries are kept up-to-date.
 * Tokens have no meaning to the cache, so effectively turn the it into a dictionary of classes.<br>
 * Eg. A token may be the location where the class was found, combined with some type of versioning data.<br>
 * If a ClassLoader stores multiple versions of the same class using the same token, only the most recent will be returned by findSharedClass.
 * <p>
 * - Security<p>
 * A SharedClassHelper will only allow classes to be stored in the cache which were defined by the ClassLoader that owns the SharedClassHelper.<br>
 * If a SecurityManager is installed, SharedClassPermissions must be used to permit read/write access to the shared class cache.
 * Permissions are granted by ClassLoader classname in the java.policy file and are fixed when the SharedClassHelper is created.<br>
 * Note also that if the createClassLoader RuntimePermission is not granted, ClassLoaders cannot be created
 * which in turn means that SharedClassHelpers cannot be created.
 * <p>
 * - Compatibility with other SharedClassHelpers<p>
 * Classes stored using the SharedClassTokenHelper cannot be retrieved using any other type of helper and vice-versa.
 * <p>
 * @author		OTI
 * @version		initial
 *
 * @see SharedClassHelper
 * @see SharedClassHelperFactory
 * @see SharedClassPermission
 */
public interface SharedClassTokenHelper extends SharedClassHelper {

	/**
	 * Find a class in the shared cache using a specific token and class name.<p>
	 * A class will be returned only for an exact String match of both the token and class name.
	 * Otherwise, null is returned.<br>
	 * To obtain an instance of the class, the byte[] returned must be passed to defineClass by the caller ClassLoader.
	 * <p>
	 * @author		OTI
	 * @version		initial
	 *
	 * @param 		token String.
	 * 					A token generated by the ClassLoader
	 *
	 * @param 		className String.
	 * 					The name of the class to be found
	 *
	 * @return		byte[].
	 * 					A byte array describing the class found, or null.
	 */
	public byte[] findSharedClass(String token, String className);

	/**
	 * Store a class in the shared cache using a specific token.<p>
	 * The class being stored must have been defined by the caller ClassLoader.<br>
	 * Returns true if the class is stored successfully or false otherwise.<br>
	 * Will return false if the class being stored was not defined by the caller ClassLoader.
	 * <p>
	 * @author		OTI
	 * @version		initial
	 *
	 * @param 		token String.
	 * 					A token generated by the ClassLoader
	 *
	 * @param 		clazz Class.
	 * 					The class to store in the shared cache
	 *
	 * @return		boolean.
	 * 					True if the class was stored successfully, false otherwise.
	 */
	public boolean storeSharedClass(String token, Class clazz);

}
